setGeneric("convert_prob",
           function(object, x, ...) standardGeneric("convert_prob"))


setMethod("convert_prob", c("ANY", "ANY"),
  function(object, x, ...) x
)


setMethod("convert_prob", c("ANY", "numeric"),
  function(object, x, ...) unname(x)
)


setMethod("convert_prob", c("BinomialVariate", "ANY"),
  function(object, x, ...) unname(drop(x))
)


setMethod("convert_prob", c("factor", "array"),
  function(object, x, ...) {
    convert_prob(object, adrop(x, length(dim(x))))
  }
)


setMethod("convert_prob", c("factor", "matrix"),
  function(object, x, ...) {
    if (nlevels(object) == 2) {
      convert_prob(object, x[, ncol(x)])
    } else {
      structure(x, dimnames = list(NULL, levels(object)))
    }
  }
)


setMethod("convert_prob", c("matrix", "array"),
  function(object, x, ...) {
    convert_prob(object, adrop(x, length(dim(x))))
  }
)


setMethod("convert_prob", c("matrix", "matrix"),
  function(object, x, ...) {
    stopifnot(ncol(object) == ncol(x))
    var_names <- colnames(x)
    if (is.null(var_names)) var_names <- colnames(object)
    if (is.null(var_names)) var_names <- paste0("y", seq_len(ncol(x)))
    structure(x, dimnames = list(NULL, var_names))
  }
)


setMethod("convert_prob", c("numeric", "array"),
  function(object, x, ...) {
    num_dim <- length(dim(x))
    x <- if (num_dim == 1) c(x) else adrop(x, num_dim)
    convert_prob(object, x)
  }
)


setMethod("convert_prob", c("numeric", "matrix"),
  function(object, x, ...) {
    stopifnot(ncol(x) == 1)
    convert_prob(object, x[, 1])
  }
)


setMethod("convert_prob", c("Surv", "matrix"),
  function(object, x, times, ...) {
    SurvProbs(x, times, distr = attr(x, "surv_distr"))
  }
)


setMethod("convert_prob", c("Surv", "numeric"),
  function(object, x, ...) {
    SurvMeans(x, distr = attr(x, "surv_distr"))
  }
)


setGeneric("convert_response",
           function(object, x, ...) standardGeneric("convert_response"))


setMethod("convert_response", c("ANY", "ANY"),
  function(object, x, ...) x
)


setMethod("convert_response", c("DiscreteVariate", "numeric"),
  function(object, x, ...) {
    x <- round(x)
    if (object@min > -Inf) x <- pmax(x, object@min)
    if (object@max < Inf) x <- pmin(x, object@max)
    new(class(object), x, min = object@min, max = object@max)
  }
)


setMethod("convert_response", c("factor", "factor"),
  function(object, x, ...) x
)


setMethod("convert_response", c("factor", "matrix"),
  function(object, x, ...) {
    factor(max.col(x), levels = seq_len(nlevels(object)),
           labels = levels(object))
  }
)


setMethod("convert_response", c("factor", "numeric"),
  function(object, x, cutoff, ...) {
    factor(x > cutoff, levels = c(FALSE, TRUE), labels = levels(object))
  }
)


setMethod("convert_response", c("integer", "numeric"),
  function(object, x, ...) {
    round(x)
  }
)


setMethod("convert_response", c("ordered", "matrix"),
  function(object, x, ...) {
    ordered(max.col(x), levels = seq_len(nlevels(object)),
            labels = levels(object))
  }
)


setMethod("convert_response", c("Surv", "SurvProbs"),
  function(object, x, cutoff, ...) {
    events <- x <= cutoff
    storage.mode(events) <- "integer"
    SurvEvents(events, x@times, distr = x@distr)
  }
)
